# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "",
    "(active)": "(mātātoa)",
    "Also available in:": "",
    "Archive": "Pūranga",
    "Atom feed": "",
    "Authors": "Kaituhi",
    "Categories": "Kāwai",
    "Comments": "Kōrerohia",
    "LANGUAGE": " Māori",
    "Languages:": "Reo",
    "More posts about %s": "",
    "Newer posts": "",
    "Next post": "",
    "Next": "Panuku",
    "No posts found.": "",
    "Nothing found.": "Kīhai i kitea",
    "Older posts": "",
    "Original site": "",
    "Posted:": "",
    "Posts about %s": "",
    "Posts by %s": "",
    "Posts for year %s": "",
    "Posts for {month_day_year}": "",
    "Posts for {month_year}": "",
    "Previous post": "",
    "Previous": "Whakamuri",
    "Publication date": "",
    "RSS feed": "Whāngai RSS",
    "Read in English": "",
    "Read more": "",
    "Skip to main content": "",
    "Source": "Matapuna",
    "Subcategories:": "",
    "Tags and Categories": "Tūtohu me nga kāwai",
    "Tags": "Tūtohu",
    "Toggle navigation": "",
    "Uncategorized": "",
    "Up": "Ki runga",
    "Updates": "Whakahou",
    "Write your page here.": "",
    "Write your post here.": "",
    "old posts, page %d": "",
    "page %d": "whārangi %d",
    "updated": "whakahoutia",
}
