# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d minuta preostalo za čitanje",
    "(active)": "(aktivno)",
    "Also available in:": "Takođe dostupan u:",
    "Archive": "Arhiva",
    "Atom feed": "",
    "Authors": "",
    "Categories": "Kategorije",
    "Comments": "Komentari",
    "LANGUAGE": "Bosanski",
    "Languages:": "Jezici:",
    "More posts about %s": "Više objava o %s",
    "Newer posts": "Novije objave",
    "Next post": "Naredna objava",
    "Next": "",
    "No posts found.": "Nema objava.",
    "Nothing found.": "Ništa nije pronađeno.",
    "Older posts": "Starije objave",
    "Original site": "Izvorni sajt",
    "Posted:": "Objavljeno:",
    "Posts about %s": "Objave o %s",
    "Posts by %s": "",
    "Posts for year %s": "Objave u godini %s",
    "Posts for {month_day_year}": "Objave za {month_day_year}",
    "Posts for {month_year}": "Objave za {month_year}",
    "Previous post": "Prethodne objave",
    "Previous": "",
    "Publication date": "Datum objavljivanja",
    "RSS feed": "RSS feed",
    "Read in English": "Pročitaj na bosanskom",
    "Read more": "Pročitaj više",
    "Skip to main content": "Preskoči na glavni sadržaj",
    "Source": "Izvor",
    "Subcategories:": "Podkategorije:",
    "Tags and Categories": "Oznake i kategorije",
    "Tags": "Oznake",
    "Toggle navigation": "",
    "Uncategorized": "",
    "Up": "",
    "Updates": "",
    "Write your page here.": "Vašu stranicu napišite ovdje.",
    "Write your post here.": "Vašu objavu napišite ovdje.",
    "old posts, page %d": "stare objave, strana %d",
    "page %d": "strana %d",
    "updated": "",
}
